@@ -12,6 +12,10 @@
 #' @param label.position the position of labels: "left", "right", "top", "bottom"
 #' @param latitude numeric vector of latitudes
 #' @param longitude numeric vector of longitudes
+#' @param density.estimation mostly logical. If TRUE, it creates a density estimation polygon. If write here "blank", it creates density estimation polygon without points.
+#' @param density.estimation.opacity a numeric vector of density estimation opacity.
+#' @param density.latitude.width bandwidths for latitude values. Defaults to normal reference bandwidth (see \link{bandwidth.nrd}).
+#' @param density.longitude.width bandwidths for longitude values. Defaults to normal reference bandwidth (see \link{bandwidth.nrd}).
 #' @param color vector of colors or palette. The color argument can be (1) a character vector of RGM or named colors; (2) the name of an RColorBrewer palette; (3) the full name of a viridis palette; (4) a function that receives a single value between 0 and 1 and returns a color. For more examples see \code{\link{colorNumeric}}
 #' @param stroke.color vector of stroke colors
 #' @param image.url character vector of URLs with an images
@@ -106,24 +110,43 @@
 #'
 #'
 #' @export
-#' @import leaflet
+#' @importFrom leaflet colorNumeric
+#' @importFrom leaflet colorFactor
+#' @importFrom leaflet leaflet
+#' @importFrom leaflet addTiles
+#' @importFrom leaflet addProviderTiles
+#' @importFrom leaflet addPolygons
+#' @importFrom leaflet addCircleMarkers
+#' @importFrom leaflet addMarkers
+#' @importFrom leaflet addLayersControl
+#' @importFrom leaflet addScaleBar
+#' @importFrom leaflet addLegend
+#' @importFrom leaflet addMiniMap
+#' @importFrom leaflet leafletOptions
+#' @importFrom leaflet labelOptions
+#' @importFrom leaflet layersControlOptions
+#' @importFrom leaflet icons
 #' @importFrom stats complete.cases
 #' @importFrom grDevices gray
-#' @importFrom grDevices rainbow
+#' @importFrom grDevices topo.colors
 #' @importFrom rowr cbind.fill
 #' @importFrom magrittr %>%
 #'
 
 map.feature <- function(languages,
-                        features = "none",
+                        features = "",
                         popup = "",
                         label = "",
-                        label.hide = FALSE,
+                        label.hide = TRUE,
                         label.fsize = 14,
                         label.position = "right",
                         stroke.features = NULL,
                         latitude = NULL,
                         longitude = NULL,
+                        density.estimation = FALSE,
+                        density.estimation.opacity = 0.2,
+                        density.longitude.width = NULL,
+                        density.latitude.width = NULL,
                         color = NULL,
                         stroke.color = NULL,
                         image.url = NULL,
@@ -160,7 +183,9 @@ map.feature <- function(languages,
          glottolog <- lingtypology::glottolog.original,
          glottolog <- lingtypology::glottolog.modified)
   if(typeof(languages) == "list"){languages <- unlist(languages)}
-  if(sum(is.glottolog(languages, response = TRUE, glottolog.source = glottolog.source)) == 0){
+  if(sum(is.glottolog(languages,
+                      response = TRUE,
+                      glottolog.source = glottolog.source)) == 0){
     stop("There is no data to map")
   }
 
@@ -171,14 +196,27 @@ map.feature <- function(languages,
     mapfeat.df$label <- label
   }
 
-  if (is.null(latitude) & is.null(longitude)) {  # if there are no latitude and longitude
-    mapfeat.df$long <- long.lang(languages, map.orientation = map.orientation, glottolog.source = glottolog.source)
+  # if there are no latitude and longitude
+
+  if (is.null(latitude) & is.null(longitude)) {
+    mapfeat.df$long <- long.lang(languages,
+                                 map.orientation = map.orientation,
+                                 glottolog.source = glottolog.source)
     mapfeat.df$lat <- lat.lang(languages, glottolog.source = glottolog.source)
   } else {   # if there are latitude and longitude
     mapfeat.df$long <- longitude
     mapfeat.df$lat <- latitude
   }
 
+  # if there are no coordinates... ------------------------------------------
+  if(sum(is.na(mapfeat.df$long&mapfeat.df$lat))> 0){
+    warning(paste("There is no coordinates for languages",
+                  paste(
+                    mapfeat.df$languages[is.na(mapfeat.df$long&mapfeat.df$lat)],
+                    collapse = ", ")),
+            call. = FALSE)
+  }
+
   # remove any rows with NAs ------------------------------------------------
   mapfeat.df <- mapfeat.df[stats::complete.cases(mapfeat.df),]
 
@@ -195,45 +233,45 @@ map.feature <- function(languages,
 
   # create a stroke dataframe -----------------------------------------------
   if(!is.null(stroke.features)){
-    mapfeat.stroke <- rowr::cbind.fill(mapfeat.df[,-2], data.frame(stroke.features))
+    mapfeat.stroke <- rowr::cbind.fill(mapfeat.df[,-2],
+                                       data.frame(stroke.features))
     mapfeat.stroke <- mapfeat.stroke[stats::complete.cases(mapfeat.stroke),]
   }
 
   # change feature names ----------------------------------------------------
   # levels(mapfeat.df$features) <- paste(names(table(mapfeat.df$features)), " (", table(mapfeat.df$features), ")", sep = "")
 
-  # if features are numeric -------------------------------------------------
-
-
   # create a palette ---------------------------------------------------------
-  if (length(table(mapfeat.df$features)) <= 1 & is.null(color)){color <- "blue"}
   if (is.null(color)) {
     if(is.numeric(mapfeat.df$features)){
-      pal <- leaflet::colorNumeric(palette = "BuPu", domain = mapfeat.df$features)
+      pal <- leaflet::colorNumeric(palette = "BuPu", domain=mapfeat.df$features)
     } else {
     set.seed(42)
-    pal <- leaflet::colorFactor(sample(grDevices::rainbow(length(unique(mapfeat.df$features))), length(unique(mapfeat.df$features))),
-                                domain = mapfeat.df$features)
+    pal <- leaflet::colorFactor(
+      sample(grDevices::rainbow(length(unique(mapfeat.df$features))),
+             length(unique(mapfeat.df$features))),
+      domain = mapfeat.df$features)
     }} else {
       if(is.numeric(mapfeat.df$features)){
-        pal <- leaflet::colorNumeric(palette = color, domain = mapfeat.df$features)
+        pal <- leaflet::colorNumeric(palette = color,domain=mapfeat.df$features)
       }else {
         pal <- leaflet::colorFactor(color, domain = mapfeat.df$features)
       }}
 
   if(!is.null(stroke.features)){
     if (is.null(stroke.color)) {
-      lev <- nlevels(as.factor(stroke.features[stats::complete.cases(stroke.features)]))
+      lev <- nlevels(as.factor(
+        stroke.features[stats::complete.cases(stroke.features)]))
       strokecolors <- grDevices::gray(lev :0 / lev)
       stroke.pal <- leaflet::colorFactor(strokecolors,
-                                         domain = mapfeat.stroke$stroke.features)
+                                        domain = mapfeat.stroke$stroke.features)
       rev.stroke.pal <- leaflet::colorFactor(rev(strokecolors),
-                                             domain = mapfeat.stroke$stroke.features)
+                                        domain = mapfeat.stroke$stroke.features)
     } else {
       stroke.pal <- leaflet::colorFactor(stroke.color,
-                                         domain = mapfeat.stroke$stroke.features)
+                                        domain = mapfeat.stroke$stroke.features)
       rev.stroke.pal <- leaflet::colorFactor(rev(stroke.color),
-                                             domain = mapfeat.stroke$stroke.features)
+                                        domain = mapfeat.stroke$stroke.features)
     }
   }
 
@@ -246,34 +284,40 @@ map.feature <- function(languages,
            warning('number of tile names (tile.name argument) is less than number of tiles (tile argument)', call. = FALSE))
   }
 
+  # create a density polygones --------------------------------------------
+  if(density.estimation != FALSE){
+    my_poly_names <- names(which(table(mapfeat.df$features) > 1))
+    my_poly <- lapply(my_poly_names, function(feature){
+      polygon.points(mapfeat.df[mapfeat.df$features == feature, 'lat'],
+                     mapfeat.df[mapfeat.df$features == feature, 'long'],
+                     latitude_width = density.latitude.width,
+                     longitude_width = density.longitude.width)
+  })
+  }
+
   ### create a map ------------------------------------------------------------
-  m <- leaflet::leaflet(mapfeat.df, option=leafletOptions(zoomControl = zoom.control)) %>%
+  m <- leaflet::leaflet(mapfeat.df, option=leaflet::leafletOptions(
+    zoomControl = zoom.control)) %>%
     leaflet::addTiles(tile[1]) %>%
     leaflet::addProviderTiles(tile[1], group = tile.name[1])
   if(length(tile) > 1){
     mapply(function(other.tiles, other.tile.names){
-      m <<- m %>% leaflet::addProviderTiles(other.tiles, group = other.tile.names)
+      m <<- m %>% leaflet::addProviderTiles(other.tiles,
+                                            group = other.tile.names)
     }, tile[-1], tile.name[-1])
   }
 
-  # map: if only one feature -------------------------------------------------
-  if (length(table(mapfeat.df$features)) <= 1){
-    m <- m %>% leaflet::addCircleMarkers(lng=mapfeat.df$long,
-                                         lat=mapfeat.df$lat,
-                                         popup= mapfeat.df$link,
-                                         label= mapfeat.df$label,
-                                         labelOptions = labelOptions(noHide = !(label.hide),
-                                                                     direction = label.position,
-                                                                     textOnly = TRUE,
-                                                                     style = list("font-size" = paste0(label.fsize, "px"))),
-                                         stroke = FALSE,
-                                         radius = radius,
-                                         color = color,
-                                         fillOpacity = opacity,
-                                         group = mapfeat.df$languages)
+  # if there is density estimation ------------------------------------------
+  if(density.estimation != FALSE){
+    lapply(seq_along(my_poly), function(x){
+      m <<- m %>% leaflet::addPolygons(data = my_poly[[x]],
+                                       color = pal(my_poly_names[x]),
+                                       opacity = 0.2,
+                                       fillOpacity=density.estimation.opacity)})
+  }
 
     # map: if there are stroke features ---------------------------------------
-  } else if(!is.null(stroke.features)){
+  if(!is.null(stroke.features)){
     m <- m %>% leaflet::addCircleMarkers(lng=mapfeat.stroke$long,
                                          lat=mapfeat.stroke$lat,
                                          popup= mapfeat.stroke$link,
@@ -281,69 +325,63 @@ map.feature <- function(languages,
                                          radius = stroke.radius*1.15,
                                          fillOpacity = stroke.opacity,
                                          color = "black") %>%
-      leaflet::addCircleMarkers(lng=mapfeat.stroke$long,
-                                lat=mapfeat.stroke$lat,
-                                popup= mapfeat.stroke$link,
-                                stroke = FALSE,
-                                radius = stroke.radius,
-                                fillOpacity = stroke.opacity,
-                                color = stroke.pal(mapfeat.stroke$stroke.features),
-                                group = mapfeat.stroke$stroke.features) %>%
-      leaflet::addCircleMarkers(lng=mapfeat.stroke$long,
-                                lat=mapfeat.stroke$lat,
-                                popup= mapfeat.df$link,
-                                label= mapfeat.df$label,
-                                labelOptions = labelOptions(noHide = !(label.hide),
-                                                            direction = label.position,
-                                                            textOnly = TRUE,
-                                                            style = list("font-size" = paste0(label.fsize, "px"))),
-                                stroke = FALSE,
-                                radius = 1.15*radius,
-                                fillOpacity = opacity,
-                                color = rev.stroke.pal(mapfeat.stroke$stroke.features),
-                                group = mapfeat.stroke$stroke.features) %>%
-      leaflet::addCircleMarkers(lng=mapfeat.df$long,
-                                lat=mapfeat.df$lat,
-                                popup= mapfeat.df$link,
-                                label= mapfeat.df$label,
-                                labelOptions = labelOptions(noHide = !(label.hide),
-                                                            direction = label.position,
-                                                            textOnly = TRUE,
-                                                            style = list("font-size" = paste0(label.fsize, "px"))),
-                                stroke = FALSE,
-                                radius = radius,
-                                fillOpacity = opacity,
-                                color = pal(mapfeat.df$features),
-                                group = mapfeat.df$features)
+leaflet::addCircleMarkers(lng=mapfeat.stroke$long,
+                  lat=mapfeat.stroke$lat,
+                  popup= mapfeat.stroke$link,
+                  stroke = FALSE,
+                  radius = stroke.radius,
+                  fillOpacity = stroke.opacity,
+                  color=stroke.pal(mapfeat.stroke$stroke.features),
+                  group = mapfeat.stroke$stroke.features) %>%
+leaflet::addCircleMarkers(lng=mapfeat.stroke$long,
+                    lat=mapfeat.stroke$lat,
+                    popup= mapfeat.df$link,
+                    label= mapfeat.df$label,
+                    labelOptions = leaflet::labelOptions(noHide = !(label.hide),
+                        direction = label.position,
+                        textOnly = TRUE,
+                        style = list("font-size" = paste0(label.fsize, "px"))),
+                    stroke = FALSE,
+                    radius = 1.15*radius,
+                    fillOpacity = opacity,
+                    color = rev.stroke.pal(mapfeat.stroke$stroke.features),
+                    group = mapfeat.stroke$stroke.features)}
 
-    # map: if there are several features ----------------------------------------
-  } else{
-    m <- m %>% leaflet::addCircleMarkers(lng=mapfeat.df$long,
+    # map: add points ----------------------------------------
+    if(density.estimation != "blank"){
+      m <- m %>% leaflet::addCircleMarkers(lng=mapfeat.df$long,
                                          lat=mapfeat.df$lat,
                                          popup= mapfeat.df$link,
-                                         label= mapfeat.df$label,
-                                         labelOptions = labelOptions(noHide = !(label.hide),
-                                                                     direction = label.position,
-                                                                     textOnly = TRUE,
-                                                                     style = list("font-size" = paste0(label.fsize, "px"))),
                                          stroke = FALSE,
-                                         radius = radius,
+                                         radius = radius*1.1,
                                          fillOpacity = opacity,
-                                         color = pal(mapfeat.df$features),
-                                         group = mapfeat.df$features)
+                                         color = "black") %>%
+      leaflet::addCircleMarkers(lng=mapfeat.df$long,
+  lat=mapfeat.df$lat,
+  popup= mapfeat.df$link,
+  label= mapfeat.df$label,
+  labelOptions = leaflet::labelOptions(noHide = !(label.hide),
+                         direction = label.position,
+                         textOnly = TRUE,
+                         style = list("font-size" = paste0(label.fsize, "px"))),
+  stroke = FALSE,
+  radius = radius,
+  fillOpacity = opacity,
+  color = pal(mapfeat.df$features),
+  group = mapfeat.df$features)
   }
 
   # map: images -------------------------------------------------------------
   if (!is.null(image.url)) {
     m <- m %>% leaflet::addMarkers(lng=mapfeat.image$long,
-                                   lat=mapfeat.image$lat,
-                                   popup= mapfeat.image$link,
-                                   icon = icons(
-                                     iconUrl = as.character(mapfeat.image$image.url),
-                                     iconWidth = image.width,
-                                     iconHeight = image.height,
-                                     iconAnchorX = - image.X.shift,
-                                     iconAnchorY = image.Y.shift))
+                             lat=mapfeat.image$lat,
+                             popup= mapfeat.image$link,
+                             icon = leaflet::icons(
+                               iconUrl = as.character(mapfeat.image$image.url),
+                               iconWidth = image.width,
+                               iconHeight = image.height,
+                               iconAnchorX = - image.X.shift,
+                               iconAnchorY = image.Y.shift))
   }
 
   # map: tile and control interaction --------------------------------------
@@ -352,16 +390,16 @@ map.feature <- function(languages,
       m <- m %>% leaflet::addLayersControl(
         baseGroups = tile.name,
         overlayGroups = mapfeat.df$features,
-        options = layersControlOptions(collapsed = FALSE))
+        options = leaflet::layersControlOptions(collapsed = FALSE))
     } else {
       m <- m %>% leaflet::addLayersControl(
         baseGroups = tile.name,
-        options = layersControlOptions(collapsed = FALSE))
+        options = leaflet::layersControlOptions(collapsed = FALSE))
     }
   } else {
     if (control == TRUE) {
       m <- m %>% leaflet::addLayersControl(overlayGroups = mapfeat.df$features,
-                                           options = layersControlOptions(collapsed = FALSE))
+                     options = leaflet::layersControlOptions(collapsed = FALSE))
     }
   }
 
@@ -373,7 +411,8 @@ map.feature <- function(languages,
 
 
   # map: legend -------------------------------------------------------------
-  if (length(table(mapfeat.df$features)) > 1 & legend == TRUE) {
+  if (sum(mapfeat.df$features == "") < length(mapfeat.df$features) &
+      legend == TRUE) {
     m <- m %>% leaflet::addLegend(title = title,
                                   position = legend.position,
                                   pal = pal,
